﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Forms;
using PartnerDataClient.Pages;

namespace PartnerDataClient
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			AddPage(new InstrumentGuidePage());
			AddPage(new InstrumentsPage());
			AddPage(new SnapshotsPage());
			AddPage(new TradesHistoryPage());
			AddPage(new SimpleOnlinePage());
			AddPage(new BulkOnlinePage());
		}

		/// <summary>
		/// Добавляет форму, как страницу на TabControl
		/// </summary>
		/// <param name="page">Форма</param>
		private void AddPage(Form page)
		{
			page.TopLevel = false;
			page.FormBorderStyle = FormBorderStyle.None;
			TabPage tbp = new TabPage { Text = page.Text };
			tabControl1.TabPages.Add(tbp);
			tbp.Controls.Add(page);
			page.WindowState = FormWindowState.Normal;
			page.Dock = DockStyle.Fill;
			page.Show();
		}

		protected override void OnClosing(CancelEventArgs e)
		{
			base.OnClosing(e);
			// Закрывает все страницы-формы, каждая из которых в свою очередь остановит свой фоновый поток
			foreach (TabPage tabPage in tabControl1.TabPages)
			{
				if (tabPage.Controls.Count == 0) continue;
				Form page = tabPage.Controls[0] as Form;
				if (page == null) continue;
				page.Close();
			}
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Close();
		}

		/// <summary>
		/// Показывает диалог для ввода имени пользователя Efir и его пароля
		/// </summary>
		private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			CredentialForm dlg = new CredentialForm { UserName = Program.UserName, Password = Program.Password };
			DialogResult result = dlg.ShowDialog(this);
			if (result == DialogResult.OK)
			{
				Program.SetCredential(dlg.UserName, dlg.Password);
			}
		}
	}
}
