﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Interfax.Efir.MarketModel;
using Interfax.Efir.PartnerDataModel;
using Interfax.Efir.PartnerDataModel.DataContracts;
using PartnerDataClient.Properties;

namespace PartnerDataClient.Pages
{
	public partial class InstrumentsPage : Form
	{
		public InstrumentsPage()
		{
			InitializeComponent();
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			// Fill combo-box with possible values of MarketFeedGroup
			List<string> groups = new List<string>(Enum.GetNames(typeof(MarketFeedGroup)));
			groups.Sort();
			foreach (var name in groups)
			{
				cbMarketFeedGroup.Items.Add(name);
			}
			cbMarketFeedGroup.SelectedItem = "MicexShares";
		}

		protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
		{
			if (_isRunning)
			{
				Debug.WriteLine("Stopping InstrumentsPage");
				_cancellation.Cancel();
				while (_isRunning)
				{
					Application.DoEvents();
				}
				Debug.WriteLine("InstrumentsPage is stopped");
			}
			base.OnClosing(e);
		}

		private void btnGet_Click(object sender, EventArgs e)
		{
			if (_isRunning)
			{
				// Stop the process
				_cancellation.Cancel();
				return;
			}

			try
			{
				SetButtonText("Stop");
				txtOutput.Text = "";
				SetStatus("Loading...");
				_cancellation = new CancellationTokenSource();
				CancellationToken token = _cancellation.Token;
				_isRunning = true;
				MarketFeedGroup marketFeedGroup = (MarketFeedGroup)Enum.Parse(typeof(MarketFeedGroup), (string)cbMarketFeedGroup.SelectedItem);
				DateTime lastUpdateDate = (dtLastUpdateDate.Checked) ? dtLastUpdateDate.Value.Date : DateTime.MinValue;

				Task.Factory.StartNew(() =>
				{
					try
					{
						// Make call to EfirPartnerDataService
						IGuideService guideSrv = Program.ProxyManager.GetProxy<IGuideService>();
						SecurityDefinitionCollection instruments = guideSrv.GetInstruments(marketFeedGroup, lastUpdateDate);
						token.ThrowIfCancellationRequested();
						SetStatus(string.Format("Total {0} secdefs are recieved", instruments.Count));
						if (instruments.Count > 0)
						{
							int nCount = 0;
							StringBuilder sb = null;
							foreach (SecurityDefinition secDef in instruments)
							{
								token.ThrowIfCancellationRequested();
								sb = sb ?? new StringBuilder();
								sb.AppendLine(secDef.ToString());
								nCount++;
								if (nCount < 100) continue;
								AppendInstrument(sb.ToString());
								sb = null;
								nCount = 0;
							}
							if (nCount > 0 && sb != null)
							{
								AppendInstrument(sb.ToString());
							}
							Debug.WriteLine("Instruments are displayed");
						}
						else
						{
							AppendInstrument("No instruments\r\n");
						}
					}
					catch (OperationCanceledException)
					{
						Debug.WriteLine("Task was cancelled");
						SetStatus("Cancelled");
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message, Resources.ErrorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
					finally
					{
						SetButtonText("Get");
						_isRunning = false;
					}
				}
				, token);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, Resources.ErrorCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private bool _isRunning;
		private CancellationTokenSource _cancellation = new CancellationTokenSource();

		private delegate void SetButtonTextCallback(string text);
		private void SetButtonText(string text)
		{
			if (btnGet.InvokeRequired)
			{
				SetButtonTextCallback d = SetButtonText;
				btnGet.Invoke(d, new object[] { text });
			}
			else
			{
				btnGet.Text = text;
			}
		}

		private delegate void AppendTextCallback(string text);
		private void AppendInstrument(string text)
		{
			if (txtOutput.InvokeRequired)
			{
				AppendTextCallback d = AppendInstrument;
				txtOutput.Invoke(d, new object[] { text });
			}
			else
			{
				txtOutput.AppendText(text);
			}
		}

		private delegate void SetStatusCallback(string text);
		private void SetStatus(string text)
		{
			if (lblStatus.InvokeRequired)
			{
				SetStatusCallback d = SetStatus;
				lblStatus.Invoke(d, new object[] { text });
			}
			else
			{
				lblStatus.Text = text;
			}
		}
	}
}
