﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Windows.Forms;
using Ifx.Helpers;
using Ifx.Helpers.ServicesFoundation.Wcf;
using Interfax.Efir.PartnerDataModel;

namespace PartnerDataClient
{
	static class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			try
			{
				string userName, password;
				CredentialsApi.ReadCred(CredentialTarget, out userName, out password);
				UserName = userName;
				Password = password;
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Cannot get credential: " + ex.Message);
			}

			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			Application.Run(new MainForm());

			CloseProxyManager();
		}

		private static IWcfProxyManager _mgr;
		private static readonly object MgrLock = new object();
		public const string CredentialTarget = "EfirPartnerDataService";
		public static string UserName { get; private set; }
		public static string Password { get; private set; }

		public static void SetCredential(string userName, string password)
		{
			if (userName == UserName && password == Password) return;
			CloseProxyManager();
			UserName = userName;
			Password = password;
			try
			{
				CredentialsApi.WriteCred(CredentialTarget, UserName, Password);
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Cannot save credential: " + ex.Message);
			}
		}

		public static IWcfProxyManager ProxyManager
		{
			get
			{
				lock (MgrLock)
				{
					if (_mgr == null)
					{
						_mgr = new PooledWcfClientManager("MainMgr");
						PooledWcfClientManager m = (PooledWcfClientManager)_mgr;
						m.AddServiceClient<IGuideService>("IGuideServiceEndpoint", 10, UserName, Password);
						m.AddServiceClient<IHistoryRecoveryService>("IHistoryRecoveryServiceEndpoint", 10, UserName, Password);
					}
					return _mgr;
				}
			}
		}

		public static void CloseProxyManager()
		{
			lock (MgrLock)
			{
				if (_mgr != null)
				{
					((IDisposable)_mgr).Dispose();
					_mgr = null;
				}
			}
		}
	}
}
